// @flow
import o from "ospec/ospec.js"
import n from "../nodemocker"

o.spec("DesktopTrayTest", () => {
	n.startGroup({
		group: __filename, allowables: [
			'path',
			'./TutanotaError',
			'./StringUtils',
			'./EntityConstants',
			'../TutanotaConstants',
			'./Utils',
			'./MapUtils',
			'./utils/Utils',
			'../EntityFunctions',
			'./utils/ArrayUtils',
			'../error/CryptoError',
			'../api/common/utils/Utils',
			'./utils/Encoding'
		]
	})


	const electron = {
		nativeImage: {
			createFromPath: path => 'this is an icon'
		},
		app: {
			callbacks: {},
			on: function (ev: string, cb: ()=>void) {
				this.callbacks[ev] = o.spy(cb)
				return this
			},
			dock: {
				show: () => {
				},
				setMenu: () => {
				},
				isVisible: () => false,
			}
		},
		Tray: n.classify({
			prototype: {
				callbacks: [],
				on: function (ev: string, cb: ()=>void) {
					this.callbacks[ev] = cb
					return this
				},
				setContextMenu: function () {
				},
				destroy: function () {
				}
			},
			statics: {}
		}),
		Menu: n.classify({
			prototype: {
				append: function () {
				}
			},
			statics: {}
		}),
		MenuItem: n.classify({
			prototype: {},
			statics: {}
		})
	}

	const conf = {
		removeListener: (key: string, cb: ()=>void) => n.spyify(conf),
		on: (key: string) => n.spyify(conf),
		getVar: (key: string) => {
			switch (key) {
				case 'runAsTrayApp':
					return false
				default:
					throw new Error(`unexpected getVar key ${key}`)
			}
		},
		setVar: (key: string) => {
		},
		getConst: (key: string) => {
			switch (key) {
				case 'iconName':
					return 'iconName.name'
				default:
					throw new Error(`unexpected getConst key ${key}`)
			}
		}
	}

	const wm = {
		getAll: () => [{getTitle: () => 'a'}, {getTitle: () => 'b'}, {getTitle: () => 'c'}]
	}

	const lang = {
		lang: {
			initialized: {
				promise: {
					then: cb => setImmediate(cb)
				}
			},
			get: (key: string) => {
				if (["openNewWindow_action", "quit_action"].includes(key)) {
					return key
				}
				throw new Error(`unexpected lang key ${key}`)
			}
		}
	}

	const notifier = {
		resolveGroupedNotification: () => {
		},
		hasNotificationsForWindow: () => false
	}

	o("update without tray", () => {
		//node modules
		const electronMock = n.mock("electron", electron).set()

		//our modules
		n.mock('../../misc/LanguageViewModel', lang).set()

		// instances
		const confMock = n.mock('__conf', conf).set()
		const notifierMock = n.mock('__notifier', notifier).set()
		const wmMock = n.mock('__wm', wm).set()

		const {DesktopTray} = n.subject('../../src/desktop/tray/DesktopTray.js')
		const tray = new DesktopTray(confMock, notifierMock)
		tray.setWindowManager(wmMock)
		electronMock.app.callbacks["ready"]()

		tray.update()
		o(confMock.getVar.callCount).equals(1)
		o(confMock.getVar.args[0]).equals('runAsTrayApp')
	})

	o("update with tray, mac, 3 windows", done => {
		n.setPlatform("darwin")
		//node modules
		const electronMock = n.mock("electron", electron).set()

		//our modules
		n.mock('../../misc/LanguageViewModel', lang).set()

		// instances
		const confMock = n.mock('__conf', conf)
		                  .with({
			                  getVar: (key: string) => {
				                  switch (key) {
					                  case 'runAsTrayApp':
						                  return true
					                  default:
						                  throw new Error(`unexpected getVar key ${key}`)
				                  }
			                  }
		                  })
		                  .set()
		const notifierMock = n.mock('__notifier', notifier).set()
		const wmMock = n.mock('__wm', wm).set()

		const {DesktopTray} = n.subject('../../src/desktop/tray/DesktopTray.js')
		const tray = new DesktopTray(confMock, notifierMock)
		tray.setWindowManager(wmMock)
		electronMock.app.callbacks["ready"]()

		tray.update()
		setTimeout(() => {
			o(electronMock.Menu.mockedInstances.length).equals(1)
			o(electronMock.MenuItem.mockedInstances.length).equals(5)
			o(electronMock.Menu.mockedInstances[0].append.callCount).equals(5)
			o(electronMock.app.dock.setMenu.callCount).equals(1)
			o(electronMock.Menu.mockedInstances.length).equals(1)
			o(electronMock.app.dock.setMenu.args[0]).equals(electronMock.Menu.mockedInstances[0])
			o(electronMock.app.dock.isVisible.callCount).equals(1)
			o(electronMock.app.dock.show.callCount).equals(1)
			done()
		}, 10)
	})

	o("update with tray, win, 3 windows", done => {
		n.setPlatform("win32")
		//node modules
		const electronMock = n.mock("electron", electron).set()

		//our modules
		n.mock('../../misc/LanguageViewModel', lang).set()

		// instances
		const confMock = n.mock('__conf', conf)
		                  .with({
			                  getVar: (key: string) => {
				                  switch (key) {
					                  case 'runAsTrayApp':
						                  return true
					                  default:
						                  throw new Error(`unexpected getVar key ${key}`)
				                  }
			                  }
		                  })
		                  .set()
		const notifierMock = n.mock('__notifier', notifier).set()
		const wmMock = n.mock('__wm', wm).set()

		const {DesktopTray} = n.subject('../../src/desktop/tray/DesktopTray.js')
		const tray = new DesktopTray(confMock, notifierMock)
		tray.setWindowManager(wmMock)
		console.log(electronMock.app.callbacks)
		electronMock.app.callbacks["ready"]()

		tray.update()
		setTimeout(() => {
			o(electronMock.Menu.mockedInstances.length).equals(1)
			o(electronMock.Menu.mockedInstances[0].append.callCount).equals(7)
			done()
		}, 10)
	})

	o("destroy tray before quit", done => {
		n.setPlatform("win32")
		//node modules
		const electronMock = n.mock("electron", electron).set()

		//our modules
		n.mock('../../misc/LanguageViewModel', lang).set()

		// instances
		const confMock = n.mock('__conf', conf)
		                  .with({
			                  getVar: (key: string) => {
				                  switch (key) {
					                  case 'runAsTrayApp':
						                  return true
					                  default:
						                  throw new Error(`unexpected getVar key ${key}`)
				                  }
			                  }
		                  })
		                  .set()
		const notifierMock = n.mock('__notifier', notifier).set()
		const wmMock = n.mock('__wm', wm).set()

		const {DesktopTray} = n.subject('../../src/desktop/tray/DesktopTray.js')
		const tray = new DesktopTray(confMock, notifierMock)
		tray.setWindowManager(wmMock)
		electronMock.app.callbacks["ready"]()

		tray.update()
		setTimeout(() => {
			electronMock.app.callbacks['will-quit']()
			o(electronMock.Tray.mockedInstances[0].destroy.callCount).equals(1)
			done()
		}, 10)
	})
})
